/*
** iplog_options.c - iplog command line argument handler.
** Copyright (C) 1999 Odin <odin@ojnk.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#ifndef HAVE_GETOPT_LONG
#	include <gnu/getopt.h>
#else
#	include <getopt.h>
#endif

#include <iplog.h>
#include <iplog_options.h>
#include <iplog_filter.h>

static const char opts[] = "FINPRSTUbcdefg:hi:kl:npr:stu:vx";

static const struct option longopts[] = {
	{"tcp",					optional_argument,	0, 10},
	{"udp",					optional_argument,	0, 11},
	{"icmp",				optional_argument,	0, 12},
	{"kill",				no_argument,		0, 'k'},
	{"user",				required_argument,	0, 'u'},
	{"group",				required_argument,	0, 'g'},
	{"rules",				required_argument,	0, 'r'},
	{"ignore",				no_argument,		0, 'd'},
	{"restart",				no_argument,		0, 'R'},
	{"logfile",				required_argument,	0, 'l'},
	{"dns-cache",			optional_argument,	0, 'c'},
	{"interface",			required_argument,	0, 'i'},
	{"get-ident",			optional_argument,	0, 'e'},
	{"tcp-resolve",			optional_argument,	0, 'T'},
	{"udp-resolve",			optional_argument,	0, 'U'},
	{"icmp-resolve",		optional_argument,	0, 'I'},
	{"disable-resolver",	optional_argument,	0, 'N'},
	{"detect-smurf",		optional_argument,	0, 'S'},
	{"detect-bogus",		optional_argument,	0, 'b'},
	{"detect-portscan",		optional_argument,	0, 'p'},
	{"detect-udp-scan",		optional_argument,	0, 'F'},
	{"detect-ping-flood",	optional_argument,	0, 'P'},
	{"detect-fin-scan",		optional_argument,	0, 'f'},
	{"detect-xmas-scan",	optional_argument,	0, 'x'},
	{"detect-null-scan",	optional_argument,	0, 'n'},
	{"detect-syn-flood",	optional_argument,	0, 's'},
	{"detect-traceroute",	optional_argument,	0, 't'},
	{"version",				no_argument,		0, 'v'},
	{"help",				no_argument,		0, 'h'},
	{NULL, 0, NULL, 0}
};

static void enable(const u_char *arg, u_long flag);
static void print_help(void);

extern u_long flags;

void get_options(int argc, char *const argv[]) {
	extern u_char *logfile, *user, *group, *ifstring, *rules;
	int opt;

	while ((opt = getopt_long(argc, argv, opts, longopts, NULL)) != EOF) {
		switch (opt) {	
			case 10:
				enable(optarg, LOG_TCP);
				break;
			case 11:
				enable(optarg, LOG_UDP);
				break;
			case 12:
				enable(optarg, LOG_ICMP);
				break;
			case 'd':
				add_dns_ignore_rules();
				break;
			case 'u':
				user = optarg;
				break;
			case 'g':
				group = optarg;
				break;
			case 'k':
				kill_iplog(15);
			case 'l':
				logfile = optarg;
				break;
			case 'c':
				enable(optarg, DNS_CACHE);
				break;
			case 'e':
				enable(optarg, GET_IDENT);
				break;
			case 'T':
				enable(optarg, TCP_RES);
				break;
			case 'U':
				enable(optarg, UDP_RES);
				break;
			case 'I':
				enable(optarg, ICMP_RES);
				break;
			case 'S':
				enable(optarg, SMURF);
				break;
			case 'b':
				enable(optarg, BOGUS);
				break;
			case 'P':
				enable(optarg, PING_FLOOD);
				break;
			case 'p':
				enable(optarg, PORTSCAN);
				break;
			case 'x':
				enable(optarg, XMAS_SCAN);
				break;
			case 'f':
				enable(optarg, FIN_SCAN);
				break;
			case 'F':
				enable(optarg, UDP_SCAN);
				break;
			case 'N':
				enable(optarg, NO_RESOLV);
				break;
			case 'n':
				enable(optarg, NULL_SCAN);
				break;
			case 's':
				enable(optarg, SYN_FLOOD);
				break;
			case 't':
				enable(optarg, TRACEROUTE);
				break;
			case 'i':
				ifstring = optarg;
				break;
			case 'r':
				rules = optarg;
				break;
			case 'R':
				kill_iplog(1);
				break;
			case 'v':
				puts("iplog version " VERSION "\nby " AUTHORS WEBPAGE);
				exit(0);
			case 'h':
			default:
				print_help();
				break;
		}
	}
}

static void enable(const u_char *arg, u_long flag) {
	if (arg == NULL || !strcasecmp(arg, "yes") || !strcasecmp(arg, "true"))
		flags |= flag;
	else if (!strcasecmp(arg, "no") || !strcasecmp(arg, "false"))
		flags &= ~flag;
	else if (!strcasecmp(arg, "toggle"))
		flags ^= flag;
	else {
		fprintf(stderr,
			"Arguments must be either \"yes\", \"true\", \"no\", \"false\" "
			"or \"toggle\".  You gave \"%s\"\n", optarg);
		print_help();
	}
}

static void print_help(void) {
	printf(
"Usage: " PACKAGE " [options] (\"*\" Denotes enabled by default)
--user      or -u <user|UID>     Run as user.
--group     or -g <group|GID>    Run with group specified.
--logfile   or -l <file>         Do not use syslog, log to <file>.
--rules     or -r <file>         Use <file> as the filter rules file.
--ignore    or -d                Ignore dns traffic from nameservers listed in
                                 /etc/resolv.conf.
--interface or -i <if0,...,ifN>  Listen on specified interface(s).
--kill      or -k                Kill iplog, if it is running.
--restart   or -R                Restart iplog, if it is running.
--help      or -h                This help screen.
--version   or -v                Print version information and exit.

--tcp[=true|false|toggle]                      %cLog tcp packets.
--udp[=true|false|toggle]                      %cLog udp packets.
--icmp[=true|false|toggle]                     %cLog icmp packets.

--dns-cache[=true|false|toggle]         or -c  %cUse dns cache.
--get-ident[=true|false|toggle]         or -e  %cGet ident info on connections
                                                to listening ports.

--tcp-resolve[=true|false|toggle]       or -T  %cResolve IPs of tcp data.
--udp-resolve[=true|false|toggle]       or -U  %cResolve IPs of udp data.
--icmp-resolve[=true|false|toggle]      or -I  %cResolve IPs of icmp data.
--disable-resolver[=true|false|toggle]  or -N  %cDo not resolve any IPs.

--detect-traceroute[=true|false|toggle] or -t  %cDetect traceroutes.
--detect-ping-flood[=true|false|toggle] or -P  %cDetect ICMP ping floods.
--detect-smurf[=true|false|toggle]      or -S  %cDetect smurf attacks.
--detect-bogus[=true|false|toggle]      or -b  %cDetect bogus tcp flags.
--detect-portscan[=true|false|toggle]   or -p  %cDetect port scans.
--detect-udp-scan[=true|false|toggle]   or -F  %cDetect UDP scans/floods.
--detect-fin-scan[=true|false|toggle]   or -f  %cDetect FIN scans.
--detect-xmas-scan[=true|false|toggle]  or -x  %cDetect Xmas scans.
--detect-null-scan[=true|false|toggle]  or -n  %cDetect null scans.
--detect-syn-flood[=true|false|toggle]  or -s  %cStop resolving IPs if a
                                                SYN flood is detected.\n",
(flags & LOG_TCP	? '*' : ' '), (flags & LOG_UDP		? '*' : ' '),
(flags & LOG_ICMP	? '*' : ' '), (flags & DNS_CACHE	? '*' : ' '),
(flags & GET_IDENT	? '*' : ' '), (flags & TCP_RES		? '*' : ' '),
(flags & UDP_RES	? '*' : ' '), (flags & ICMP_RES		? '*' : ' '),
(flags & NO_RESOLV	? '*' : ' '), (flags & TRACEROUTE	? '*' : ' '),
(flags & PING_FLOOD	? '*' : ' '), (flags & SMURF		? '*' : ' '),
(flags & BOGUS		? '*' : ' '), (flags & PORTSCAN		? '*' : ' '),
(flags & UDP_SCAN	? '*' : ' '), (flags & FIN_SCAN		? '*' : ' '),
(flags & XMAS_SCAN	? '*' : ' '), (flags & NULL_SCAN	? '*' : ' '),
(flags & SYN_FLOOD	? '*' : ' '));
	exit(0);
}

/* vim:ts=4:sw=8:tw=0 */
